@@ -295,6 +295,16 @@ mcp_dispatch_operation_end_delay (McpDispatchOperation *self,
  *
  * Leave all channels in this bundle by using RemoveMembersWithReason if the
  * channel has the Group interface, or Close if not.
+ *
+ * This method was intended for StreamedMedia channels, which (ab)used the
+ * Group interface for call control. StreamedMedia channels have been
+ * superseded by Call channels, which have a proper "hang up" method which
+ * should be used instead.
+ *
+ * Deprecated: 5.15.1: Use tp_call_channel_hangup_async() to
+ *  hang up Call channels, mcp_dispatch_operation_close_channels() to close
+ *  generic channels, or mcp_dispatch_operation_destroy_channels() to
+ *  terminate the channel destructively.
  */
 void
 mcp_dispatch_operation_leave_channels (McpDispatchOperation *self,
@@ -320,6 +330,9 @@ mcp_dispatch_operation_leave_channels (McpDispatchOperation *self,
  *  (usually recommended), wait for Observers to reply first
  *
  * Close all channels in this bundle by using the Close D-Bus method.
+ *
+ * Plugins that terminate an audio or audio/video call should
+ * use tp_call_channel_hangup_async() instead.
  */
 void
 mcp_dispatch_operation_close_channels (McpDispatchOperation *self,
@@ -340,6 +353,9 @@ mcp_dispatch_operation_close_channels (McpDispatchOperation *self,
  *
  * Close all channels in this bundle destructively, by using the Destroy D-Bus
  * method if implemented, or the Close D-Bus method if not.
+ *
+ * Plugins that terminate an audio or audio/video call should
+ * use tp_call_channel_hangup_async() instead.
  */
 void
 mcp_dispatch_operation_destroy_channels (McpDispatchOperation *self,
@@ -425,10 +441,13 @@ mcp_dispatch_operation_find_channel_by_type (McpDispatchOperation *self,
 
           if (ret_ref_channel != NULL)
             {
+              /* FIXME: in next, this method should take a TpClientFactory
+               * argument, and pass it on here */
               TpConnection *connection =
                 mcp_dispatch_operation_ref_connection (self);
 
-              *ret_ref_channel = tp_channel_new_from_properties (connection,
+              *ret_ref_channel = tp_simple_client_factory_ensure_channel (
+                  tp_proxy_get_factory (connection), connection,
                   channel_path, properties, NULL);
 
               g_object_unref (connection);
@@ -464,7 +483,13 @@ mcp_dispatch_operation_ref_connection (McpDispatchOperation *self)
 
   if (conn_path != NULL && dbus != NULL)
     {
-      connection = tp_connection_new (dbus, NULL, conn_path, NULL);
+      /* FIXME: in next, this method should take a TpClientFactory argument
+       * instead of making a new one here */
+      TpSimpleClientFactory *factory = tp_simple_client_factory_new (dbus);
+
+      connection = tp_simple_client_factory_ensure_connection (factory,
+          conn_path, NULL, NULL);
+      g_object_unref (factory);
     }
 
   g_object_unref (dbus);
@@ -486,6 +511,8 @@ TpChannel *
 mcp_dispatch_operation_ref_nth_channel (McpDispatchOperation *self,
     guint n)
 {
+  /* FIXME: in next, this method should take a TpClientFactory argument,
+   * and pass it on here */
   TpConnection *connection = mcp_dispatch_operation_ref_connection (self);
   GHashTable *channel_properties = NULL;
   const gchar *channel_path = NULL;
@@ -505,10 +532,9 @@ mcp_dispatch_operation_ref_nth_channel (McpDispatchOperation *self,
   if (channel_properties == NULL)
     goto finally;
 
-  channel = tp_channel_new_from_properties (connection,
-      channel_path,
-      channel_properties,
-      NULL);
+  channel = tp_simple_client_factory_ensure_channel (
+      tp_proxy_get_factory (connection),
+      connection, channel_path, channel_properties, NULL);
 
 finally:
   tp_clear_object (&connection);
