@@ -3,10 +3,8 @@
 /*
  * This file is part of mission-control
  *
- * Copyright (C) 2007-2009 Nokia Corporation.
- * Copyright (C) 2009 Collabora Ltd.
- *
- * Contact: Naba Kumar  <naba.kumar@nokia.com>
+ * Copyright © 2007-2011 Nokia Corporation.
+ * Copyright © 2009-2011 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -49,9 +47,6 @@ G_GNUC_INTERNAL void _mcd_connection_start_dispatching (McdConnection *self,
 
 G_GNUC_INTERNAL gboolean _mcd_connection_is_ready (McdConnection *self);
 
-G_GNUC_INTERNAL void _mcd_connection_set_nickname (McdConnection *self,
-                                                   const gchar *nickname);
-
 G_GNUC_INTERNAL void _mcd_connection_set_avatar (McdConnection *self,
                                                  const GArray *avatar,
                                                  const gchar *mime_type);
@@ -60,6 +55,15 @@ G_GNUC_INTERNAL void _mcd_connection_update_client_caps (McdConnection *self,
 
 G_GNUC_INTERNAL gboolean _mcd_connection_presence_info_is_ready (McdConnection *self);
 
+G_GNUC_INTERNAL void _mcd_connection_take_emergency_numbers (McdConnection *self,
+    GSList *numbers);
+
+G_GNUC_INTERNAL gboolean _mcd_connection_target_id_is_urgent (McdConnection *connection,
+    const gchar *name);
+
+G_GNUC_INTERNAL gboolean _mcd_connection_target_handle_is_urgent (McdConnection *self,
+    guint handle);
+
 G_END_DECLS
 
 #endif
