@@ -32,12 +32,6 @@
 G_BEGIN_DECLS
 
 G_GNUC_INTERNAL
-gboolean _mcd_master_account_replace_transport (McdMaster *master,
-                                                McdAccount *account);
-
-void _mcd_master_get_nth_account_connection (McdMaster *master, gint i,
-                                             McdAccountConnectionFunc *func,
-                                             gpointer *userdata);
 
 McdManager *_mcd_master_lookup_manager (McdMaster *master,
                                         const gchar *unique_name);
