@@ -44,9 +44,6 @@ def test(q, bus, mc):
         properties.get('InvalidAccounts')
     interfaces = properties.get('Interfaces')
 
-    # assert that current functionality exists
-    assert cs.AM_IFACE_NOKIA_QUERY in interfaces, interfaces
-
     params = dbus.Dictionary({"account": "someguy@example.com",
         "password": "secrecy"}, signature='sv')
     (cm_name_ref, account) = create_fakecm_account(q, bus, mc, params)
@@ -86,7 +83,6 @@ def test(q, bus, mc):
 
     interfaces = properties.get('Interfaces')
     assert cs.ACCOUNT_IFACE_AVATAR in interfaces, interfaces
-    assert cs.ACCOUNT_IFACE_NOKIA_COMPAT in interfaces, interfaces
     assert cs.ACCOUNT_IFACE_NOKIA_CONDITIONS in interfaces, interfaces
 
     # sanity check
@@ -103,6 +99,22 @@ def test(q, bus, mc):
             signal='AccountPropertyChanged',
             interface=cs.ACCOUNT,
             args=[{'DisplayName': 'Work account'}]),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute',
+            args=[account_path, 'DisplayName', 'Work account']),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):],
+                {'DisplayName': 'Work account'},
+                {'DisplayName': 0}, # flags
+                []],
+            ),
         EventPattern('dbus-return', method='Set'),
         )
     assert account_props.Get(cs.ACCOUNT, 'DisplayName') == 'Work account'
@@ -114,6 +126,22 @@ def test(q, bus, mc):
             signal='AccountPropertyChanged',
             interface=cs.ACCOUNT,
             args=[{'Icon': 'im-jabber'}]),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute',
+            args=[account_path, 'Icon', 'im-jabber']),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):],
+                {'Icon': 'im-jabber'},
+                {'Icon': 0}, # flags
+                []],
+            ),
         EventPattern('dbus-return', method='Set'),
         )
     assert account_props.Get(cs.ACCOUNT, 'Icon') == 'im-jabber'
@@ -127,37 +155,47 @@ def test(q, bus, mc):
             interface=cs.ACCOUNT,
             args=[{'Nickname': 'Joe Bloggs'}]),
         EventPattern('dbus-return', method='Set'),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute',
+            args=[account_path, 'Nickname', 'Joe Bloggs']),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):],
+                {'Nickname': 'Joe Bloggs'},
+                {'Nickname': 0}, # flags
+                []],
+            ),
         )
     assert account_props.Get(cs.ACCOUNT, 'Nickname') == 'Joe Bloggs'
 
-    call_async(q, dbus.Interface(account, cs.ACCOUNT_IFACE_NOKIA_COMPAT),
-            'SetHasBeenOnline')
-    q.expect_many(
-        EventPattern('dbus-signal',
-            path=account_path,
-            signal='AccountPropertyChanged',
-            interface=cs.ACCOUNT,
-            args=[{'HasBeenOnline': True}]),
-        EventPattern('dbus-return', method='SetHasBeenOnline'),
-        )
-    assert account_props.Get(cs.ACCOUNT, 'HasBeenOnline') == True
-
-    call_async(q, account_props, 'Set', cs.ACCOUNT_IFACE_NOKIA_COMPAT,
-            'SecondaryVCardFields',
-            ['x-badger', 'x-mushroom'])
-    # there's no change notification for the Compat properties
-    q.expect_many(
-        EventPattern('dbus-return', method='Set'),
-        )
-    assert account_props.Get(cs.ACCOUNT_IFACE_NOKIA_COMPAT,
-            'SecondaryVCardFields') == ['x-badger', 'x-mushroom']
-
     call_async(q, account_props, 'Set', cs.ACCOUNT_IFACE_NOKIA_CONDITIONS,
             'Condition',
             dbus.Dictionary({':foo': 'bar'}, signature='ss'))
     # there's no change notification for the Condition
     q.expect_many(
         EventPattern('dbus-return', method='Set'),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute',
+            args=[account_path, 'condition-:foo', 'bar']),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):],
+                {'condition-:foo': 'bar'},
+                {'condition-:foo': 0}, # flags
+                []],
+            ),
         )
     assert account_props.Get(cs.ACCOUNT_IFACE_NOKIA_CONDITIONS,
             'Condition') == {':foo': 'bar'}
@@ -174,6 +212,23 @@ def test(q, bus, mc):
             interface=cs.ACCOUNT,
             args=[{'Supersedes': [cs.ACCOUNT_PATH_PREFIX + 'x/y/z']}]),
         EventPattern('dbus-return', method='Set'),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute',
+            args=[account_path, 'Supersedes',
+                [cs.ACCOUNT_PATH_PREFIX + 'x/y/z']]),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):],
+                {'Supersedes': [cs.ACCOUNT_PATH_PREFIX + 'x/y/z']},
+                {'Supersedes': 0}, # flags
+                []],
+            ),
         )
     assertEquals(dbus.Array([cs.ACCOUNT_PATH_PREFIX + 'x/y/z'],
                 signature='o'),
@@ -183,6 +238,16 @@ def test(q, bus, mc):
     # but is a no-op, although in future it should change to raising an
     # exception
 
+    forbidden = [
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringSetAttribute'),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='UpdateAttributes'),
+       ]
+    q.forbid_events(forbidden)
+
     # this variable's D-Bus type must differ from the types of all known
     # properties
     badly_typed = dbus.Struct(('wrongly typed',), signature='s')
@@ -206,15 +271,6 @@ def test(q, bus, mc):
         else:
             raise AssertionError('Setting %s with wrong type should fail' % p)
 
-    for p in ('SecondaryVCardFields'):
-        try:
-            account_props.Set(cs.ACCOUNT_IFACE_NOKIA_COMPAT, p, badly_typed)
-        except dbus.DBusException, e:
-            assert e.get_dbus_name() == cs.INVALID_ARGUMENT, \
-                    (p, e.get_dbus_name())
-        else:
-            raise AssertionError('Setting %s with wrong type should fail' % p)
-
     for p in ('Condition',):
         try:
             account_props.Set(cs.ACCOUNT_IFACE_NOKIA_CONDITIONS, p,
@@ -233,12 +289,13 @@ def test(q, bus, mc):
     assert properties['Icon'] == 'im-jabber'
     properties = account_props.GetAll(cs.ACCOUNT_IFACE_AVATAR)
     assert properties['Avatar'] == ([], '')
-    properties = account_props.GetAll(cs.ACCOUNT_IFACE_NOKIA_COMPAT)
-    assert properties['SecondaryVCardFields'] == ['x-badger', 'x-mushroom']
+
+    q.unforbid_events(forbidden)
 
     # Delete the account
-    assert account_iface.Remove() is None
-    account_event, account_manager_event = q.expect_many(
+    call_async(q, account_iface, 'Remove')
+    q.expect_many(
+        EventPattern('dbus-return', method='Remove'),
         EventPattern('dbus-signal',
             path=account_path,
             signal='Removed',
@@ -251,6 +308,18 @@ def test(q, bus, mc):
             interface=cs.AM,
             args=[account_path]
             ),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='DeferringDelete',
+            args=[account_path]),
+        EventPattern('dbus-signal',
+            interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+            signal='CommittingOne',
+            args=[account_path]),
+        EventPattern('dbus-method-call',
+            interface=cs.TEST_DBUS_ACCOUNT_SERVICE_IFACE,
+            method='DeleteAccount',
+            args=[account_path[len(cs.ACCOUNT_PATH_PREFIX):]]),
         )
 
     # Check the account is correctly deleted
