@@ -22,17 +22,49 @@ import dbus
 import dbus.service
 
 from servicetest import call_async, assertEquals
-from mctest import exec_test, SimulatedClient, \
-        create_fakecm_account, enable_fakecm_account, SimulatedChannel, \
-        expect_client_setup
+from mctest import (
+    exec_test, SimulatedClient,
+    create_fakecm_account, enable_fakecm_account, SimulatedChannel,
+    expect_client_setup,
+    ChannelDispatcher, ChannelDispatchOperation, ChannelRequest)
 import constants as cs
 
-def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
-    # Now gnome-shell wants to give the channel to another handle
-    gs_cd = bus.get_object(cs.CD, cs.CD_PATH)
-    gs_cd_iface = dbus.Interface(gs_cd, cs.CD)
+REQUEST = dbus.Dictionary({
+        cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_TEXT,
+        cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
+        cs.TARGET_ID: 'juliet',
+        }, signature='sv')
 
-    call_async(q, gs_cd_iface, 'DelegateChannels',
+def test_ensure(q, bus, account, conn, chan, expected_handler_path):
+    """Tests that a client Ensure-ing the channel causes HandleChannels to be
+    called on the current handler. (Previously, DelegateChannels() and
+    PresentChannel() both broke this.)"""
+    cd = ChannelDispatcher(bus)
+    call_async(q, cd, 'EnsureChannel', account.object_path, REQUEST, 0, '')
+    e = q.expect('dbus-return', method='EnsureChannel')
+
+    cr = ChannelRequest(bus, e.value[0])
+    cr.Proceed()
+
+    e = q.expect('dbus-method-call', interface=cs.CONN_IFACE_REQUESTS,
+        method='EnsureChannel',
+        path=conn.object_path, args=[REQUEST], handled=False)
+    q.dbus_return(e.message, False,
+        chan.object_path, chan.immutable, signature='boa{sv}')
+
+    e = q.expect('dbus-method-call',
+            interface=cs.HANDLER, method='HandleChannels',
+            handled=False)
+    assertEquals(expected_handler_path, e.path)
+    q.dbus_return(e.message, signature='')
+
+def test_delegate_channel(q, bus, mc, account, conn, chan, empathy, empathy_bus, gs):
+    # Test that re-Ensure-ing works before we start Delegating and Presenting.
+    test_ensure(q, bus, account, conn, chan, gs.object_path)
+
+    # Now gnome-shell wants to give the channel to another handler
+    gs_cd = ChannelDispatcher(bus)
+    call_async(q, gs_cd, 'DelegateChannels',
         [chan.object_path], 0, "")
 
     # Empathy is asked to handle the channel and accept
@@ -46,11 +78,14 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
     e = q.expect('dbus-return', method='DelegateChannels')
     assertEquals(([chan.object_path], {}), e.value)
 
-    # Let's play ping-pong channel! Empathy give the channel back to GS
-    emp_cd = empathy_bus.get_object(cs.CD, cs.CD_PATH)
-    emp_cd_iface = dbus.Interface(emp_cd, cs.CD)
+    # Test that re-Ensure-ing the channel still works, and sends it to
+    # the right place.
+    test_ensure(q, bus, account, conn, chan, empathy.object_path)
 
-    call_async(q, emp_cd_iface, 'DelegateChannels',
+    # Let's play ping-pong with the channel! Empathy gives the channel
+    # back to GS
+    emp_cd = ChannelDispatcher(empathy_bus)
+    call_async(q, emp_cd, 'DelegateChannels',
         [chan.object_path], 0, "")
 
     # gnome-shell is asked to handle the channel and accept
@@ -64,8 +99,12 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
     e = q.expect('dbus-return', method='DelegateChannels')
     assertEquals(([chan.object_path], {}), e.value)
 
+    # Test that re-Ensure-ing the channel sttill works, and sends it
+    # to the right place.
+    test_ensure(q, bus, account, conn, chan, gs.object_path)
+
     # gnome-shell wants to give it back, again
-    call_async(q, gs_cd_iface, 'DelegateChannels',
+    call_async(q, gs_cd, 'DelegateChannels',
         [chan.object_path], 0, "")
 
     # Empathy is asked to handle the channel but refuses
@@ -80,14 +119,18 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
     e = q.expect('dbus-return', method='DelegateChannels')
     assertEquals(([], {chan.object_path: (cs.NOT_AVAILABLE, 'No thanks')}), e.value)
 
+    # Test that re-Ensure-ing the channel sttill works, and sends it
+    # to the right place.
+    test_ensure(q, bus, account, conn, chan, gs.object_path)
+
     # Empathy doesn't handle the channel atm but tries to delegates it
-    call_async(q, emp_cd_iface, 'DelegateChannels',
+    call_async(q, emp_cd, 'DelegateChannels',
         [chan.object_path], 0, "")
 
     q.expect('dbus-error', method='DelegateChannels', name=cs.NOT_YOURS)
 
     # gnome-shell which is handling the channel asks to re-ensure it
-    call_async(q, gs_cd_iface, 'PresentChannel',
+    call_async(q, gs_cd, 'PresentChannel',
         chan.object_path, 0)
 
     # gnome-shell is asked to re-handle the channel
@@ -101,7 +144,7 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
     q.expect('dbus-return', method='PresentChannel')
 
     # empathy which is not handling the channel asks to re-ensure it
-    call_async(q, emp_cd_iface, 'PresentChannel',
+    call_async(q, emp_cd, 'PresentChannel',
         chan.object_path, 0)
 
     # gnome-shell is asked to re-handle the channel
@@ -114,6 +157,10 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
 
     q.expect('dbus-return', method='PresentChannel')
 
+    # Test that re-Ensure-ing the channel *still* works, and sends it
+    # to the right place.
+    test_ensure(q, bus, account, conn, chan, gs.object_path)
+
     # Empathy crashes
     empathy.release_name()
 
@@ -124,7 +171,7 @@ def test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs):
             )
 
     # gnome-shell wants to delegate, but there is no other handler
-    call_async(q, gs_cd_iface, 'DelegateChannels',
+    call_async(q, gs_cd, 'DelegateChannels',
         [chan.object_path], 0, "")
 
     e = q.expect('dbus-return', method='DelegateChannels')
@@ -160,7 +207,7 @@ def test(q, bus, mc):
     # wait for MC to download the properties
     expect_client_setup(q, [empathy, gs])
 
-    cd = bus.get_object(cs.CD, cs.CD_PATH)
+    cd = ChannelDispatcher(bus)
 
     # incoming text channel
     channel_properties = dbus.Dictionary(text_fixed_properties,
@@ -184,13 +231,12 @@ def test(q, bus, mc):
 
     channels, cdo_path,props = e.args
 
-    cdo = bus.get_object(cs.CD, cdo_path)
-    cdo_iface = dbus.Interface(cdo, cs.CDO)
+    cdo = ChannelDispatchOperation(bus, cdo_path)
 
     q.dbus_return(e.message, signature='')
 
     # gnome-shell handles the channel itself first
-    call_async(q, cdo_iface, 'HandleWith',
+    call_async(q, cdo, 'HandleWith',
             cs.tp_name_prefix + '.Client.GnomeShell')
 
     e = q.expect('dbus-method-call',
@@ -201,7 +247,7 @@ def test(q, bus, mc):
     q.dbus_return(e.message, signature='')
 
     # test delegating an incoming channel
-    test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs)
+    test_delegate_channel(q, bus, mc, account, conn, chan, empathy, empathy_bus, gs)
 
     # Empathy is back
     empathy = SimulatedClient(q, empathy_bus, 'EmpathyChat',
@@ -210,27 +256,21 @@ def test(q, bus, mc):
     expect_client_setup(q, [empathy])
 
     # gnome-shell requests a channel for itself
-    request = dbus.Dictionary({
-            cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_TEXT,
-            cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
-            cs.TARGET_ID: 'juliet',
-            }, signature='sv')
-
     call_async(q, cd, 'CreateChannelWithHints',
-            account.object_path, request, 0,
+            account.object_path, REQUEST, 0,
             cs.tp_name_prefix + '.Client.GnomeShell',
-            {}, dbus_interface=cs.CD)
+            {})
     e = q.expect('dbus-return', method='CreateChannelWithHints')
 
-    cr = bus.get_object(cs.AM, e.value[0])
-    cr.Proceed(dbus_interface=cs.CR)
+    cr = ChannelRequest(bus, e.value[0])
+    cr.Proceed()
 
     e = q.expect('dbus-method-call', interface=cs.CONN_IFACE_REQUESTS,
         method='CreateChannel',
-        path=conn.object_path, args=[request], handled=False)
+        path=conn.object_path, args=[REQUEST], handled=False)
 
     # channel is created
-    chan = SimulatedChannel(conn, request)
+    chan = SimulatedChannel(conn, REQUEST)
 
     q.dbus_return(e.message,
         chan.object_path, chan.immutable, signature='oa{sv}')
@@ -245,7 +285,7 @@ def test(q, bus, mc):
     q.dbus_return(e.message, signature='')
 
     # test delegating an outgoing channel
-    test_delegate_channel(q, bus, mc, account, chan, empathy, empathy_bus, gs)
+    test_delegate_channel(q, bus, mc, account, conn, chan, empathy, empathy_bus, gs)
 
 if __name__ == '__main__':
     exec_test(test, {})
