@@ -33,6 +33,7 @@ from twisted.internet import reactor
 import dbus
 import dbus.service
 
+from fakeaccountsservice import FakeAccountsService
 from fakeconnectivity import FakeConnectivity
 
 def install_colourer():
@@ -104,7 +105,8 @@ class MC(dbus.proxies.ProxyObject):
         return events[3:]
 
 def exec_test_deferred (fun, params, protocol=None, timeout=None,
-        preload_mc=True, initially_online=True):
+        preload_mc=True, initially_online=True, use_fake_accounts_service=True,
+        pass_kwargs=False):
     colourer = None
 
     if sys.stdout.isatty():
@@ -129,8 +131,24 @@ def exec_test_deferred (fun, params, protocol=None, timeout=None,
     else:
         mc = None
 
+    if use_fake_accounts_service:
+        fake_accounts_service = FakeAccountsService(queue, bus)
+
+        if preload_mc:
+            queue.expect('dbus-signal',
+                    path=cs.TEST_DBUS_ACCOUNT_PLUGIN_PATH,
+                    interface=cs.TEST_DBUS_ACCOUNT_PLUGIN_IFACE,
+                    signal='Active')
+    else:
+        fake_accounts_service = None
+
+    if pass_kwargs:
+        kwargs=dict(fake_accounts_service=fake_accounts_service)
+    else:
+        kwargs=dict()
+
     try:
-        fun(queue, bus, mc)
+        fun(queue, bus, mc, **kwargs)
     except Exception, e:
         import traceback
         traceback.print_exc()
@@ -185,9 +203,10 @@ def exec_test_deferred (fun, params, protocol=None, timeout=None,
       sys.stdout = colourer.fh
 
 def exec_test(fun, params=None, protocol=None, timeout=None,
-              preload_mc=True, initially_online=True):
+              preload_mc=True, initially_online=True,
+              use_fake_accounts_service=True, pass_kwargs=False):
   reactor.callWhenRunning (exec_test_deferred, fun, params, protocol, timeout,
-          preload_mc, initially_online)
+          preload_mc, initially_online, use_fake_accounts_service, pass_kwargs)
   reactor.run()
 
 class SimulatedConnection(object):
@@ -202,12 +221,18 @@ class SimulatedConnection(object):
         return self._last_handle
 
     def __init__(self, q, bus, cmname, protocol, account_part, self_ident,
+            self_alias=None,
             implement_get_interfaces=True, has_requests=True,
             has_presence=False, has_aliasing=False, has_avatars=False,
-            avatars_persist=True, extra_interfaces=[], has_hidden=False):
+            avatars_persist=True, extra_interfaces=[], has_hidden=False,
+            implement_get_aliases=True, initial_avatar=None,
+            server_delays_avatar=False):
         self.q = q
         self.bus = bus
 
+        if self_alias is None:
+            self_alias = self_ident
+
         self.bus_name = '.'.join([cs.tp_name_prefix, 'Connection',
                 cmname, protocol.replace('-', '_'), account_part])
         self._bus_name_ref = dbus.service.BusName(self.bus_name, self.bus)
@@ -219,6 +244,7 @@ class SimulatedConnection(object):
         self.status = cs.CONN_STATUS_DISCONNECTED
         self.reason = cs.CONN_STATUS_CONNECTING
         self.self_ident = self_ident
+        self.self_alias = self_alias
         self.self_handle = self.ensure_handle(cs.HT_CONTACT, self_ident)
         self.channels = []
         self.has_requests = has_requests
@@ -227,8 +253,10 @@ class SimulatedConnection(object):
         self.has_avatars = has_avatars
         self.avatars_persist = avatars_persist
         self.extra_interfaces = extra_interfaces[:]
+        self.avatar_delayed = server_delays_avatar
 
         self.interfaces = []
+        self.interfaces.append(cs.CONN_IFACE_CONTACTS)
 
         if self.has_requests:
             self.interfaces.append(cs.CONN_IFACE_REQUESTS)
@@ -241,9 +269,11 @@ class SimulatedConnection(object):
         if self.extra_interfaces:
             self.interfaces.extend(self.extra_interfaces)
 
-        if self.avatars_persist:
+        if initial_avatar is not None:
+            self.avatar = initial_avatar
+        elif self.avatars_persist:
             self.avatar = dbus.Struct((dbus.ByteArray('my old avatar'),
-                'text/plain'), signature='ays')
+                    'text/plain'), signature='ays')
         else:
             self.avatar = None
 
@@ -267,6 +297,9 @@ class SimulatedConnection(object):
                     path=self.object_path, interface=cs.CONN,
                     method='GetInterfaces')
 
+        q.add_dbus_method_impl(self.RequestHandles,
+                path=self.object_path, interface=cs.CONN,
+                method='RequestHandles')
         q.add_dbus_method_impl(self.InspectHandles,
                 path=self.object_path, interface=cs.CONN,
                 method='InspectHandles')
@@ -281,6 +314,13 @@ class SimulatedConnection(object):
         q.add_dbus_method_impl(self.GetContactAttributes,
                 path=self.object_path,
                 interface=cs.CONN_IFACE_CONTACTS, method='GetContactAttributes')
+        q.add_dbus_method_impl(self.GetContactByID,
+                path=self.object_path,
+                interface=cs.CONN_IFACE_CONTACTS, method='GetContactByID')
+        q.add_dbus_method_impl(self.Get_ContactAttributeInterfaces,
+                path=self.object_path,
+                interface=cs.PROPERTIES_IFACE, method='Get',
+                args=[cs.CONN_IFACE_CONTACTS, 'ContactAttributeInterfaces'])
         q.add_dbus_method_impl(self.GetAll_Contacts,
                 path=self.object_path,
                 interface=cs.PROPERTIES_IFACE, method='GetAll',
@@ -313,6 +353,11 @@ class SimulatedConnection(object):
                     method='GetAliasFlags',
                     args=[])
 
+            if implement_get_aliases:
+                q.add_dbus_method_impl(self.GetAliases,
+                        path=self.object_path,
+                        interface=cs.CONN_IFACE_ALIASING, method='GetAliases')
+
         if has_avatars:
             q.add_dbus_method_impl(self.GetAvatarRequirements,
                     path=self.object_path, interface=cs.CONN_IFACE_AVATARS,
@@ -341,8 +386,22 @@ class SimulatedConnection(object):
         if has_hidden:
             self.statuses['hidden'] = (cs.PRESENCE_TYPE_HIDDEN, True, True)
 
-        self.presence = dbus.Struct((cs.PRESENCE_TYPE_OFFLINE, 'offline', ''),
-                signature='uss')
+        # "dbus.UInt32" to work around
+        # https://bugs.freedesktop.org/show_bug.cgi?id=69967
+        self.presence = dbus.Struct((dbus.UInt32(cs.PRESENCE_TYPE_OFFLINE),
+            'offline', ''), signature='uss')
+
+    def change_self_ident(self, ident):
+        self.self_ident = ident
+        self.self_handle = self.ensure_handle(cs.HT_CONTACT, ident)
+        self.q.dbus_emit(self.object_path, cs.CONN, 'SelfHandleChanged',
+                self.self_handle, signature='u')
+
+    def change_self_alias(self, alias):
+        self.self_alias = alias
+        self.q.dbus_emit(self.object_path, cs.CONN_IFACE_ALIASING,
+                'AliasesChanged', [(self.self_handle, self.self_alias)],
+                signature='a(us)')
 
     def release_name(self):
         del self._bus_name_ref
@@ -357,11 +416,19 @@ class SimulatedConnection(object):
 
     def forget_avatar(self):
         self.avatar = (dbus.ByteArray(''), '')
+        self.avatar_delayed = False
 
     # not actually very relevant for MC so hard-code 0 for now
     def GetAliasFlags(self, e):
         self.q.dbus_return(e.message, 0, signature='u')
 
+    def GetAliases(self, e):
+        ret = dbus.Dictionary(signature='us')
+        if self.self_handle in e.args[0]:
+            ret[self.self_handle] = self.self_alias
+
+        self.q.dbus_return(e.message, ret, signature='a{us}')
+
     # mostly for the UI's benefit; for now hard-code the requirements from XMPP
     def GetAvatarRequirements(self, e):
         self.q.dbus_return(e.message, ['image/jpeg'], 0, 0, 96, 96, 8192,
@@ -384,7 +451,25 @@ class SimulatedConnection(object):
 
         # the user has an avatar already, if they persist; nobody else does
         if self.self_handle in e.args[0]:
-            if self.avatar is not None:
+            if self.avatar is None:
+                # GetKnownAvatarTokens has the special case that "where
+                # the avatar does not persist between connections, a CM
+                # should omit the self handle from the returned map until
+                # an avatar is explicitly set or cleared". We'd have been
+                # better off with a more explicit design, but it's too
+                # late now...
+                assert not self.avatars_persist
+            else:
+                # "a CM must always have the tokens for the self handle
+                # if one is set (even if it is set to no avatar)"
+                # so behave as though we'd done a network round-trip to
+                # check what our token was, and found our configured
+                # token
+                if self.avatar_delayed:
+                    self.q.dbus_emit(self.object_path, cs.CONN_IFACE_AVATARS,
+                            'AvatarUpdated', self.self_handle,
+                            str(self.avatar[0]), signature='us')
+
                 # we just stringify the avatar as the token
                 # (also, empty avatar => no avatar => empty token)
                 ret[self.self_handle] = str(self.avatar[0])
@@ -393,6 +478,7 @@ class SimulatedConnection(object):
 
     def SetAvatar(self, e):
         self.avatar = dbus.Struct(e.args, signature='ays')
+        self.avatar_delayed = False
 
         # we just stringify the avatar as the token
         self.q.dbus_return(e.message, str(self.avatar[0]), signature='s')
@@ -415,7 +501,9 @@ class SimulatedConnection(object):
 
     def SetPresence(self, e):
         if e.args[0] in self.statuses:
-            presence = dbus.Struct((self.statuses[e.args[0]][0],
+            # "dbus.UInt32" to work around
+            # https://bugs.freedesktop.org/show_bug.cgi?id=69967
+            presence = dbus.Struct((dbus.UInt32(self.statuses[e.args[0]][0]),
                     e.args[0], e.args[1]), signature='uss')
 
             old_presence = self.presence
@@ -423,11 +511,10 @@ class SimulatedConnection(object):
             if presence != old_presence:
                 self.presence = presence
 
-                if self.status == cs.CONN_STATUS_CONNECTED:
-                    self.q.dbus_emit(self.object_path,
-                            cs.CONN_IFACE_SIMPLE_PRESENCE, 'PresencesChanged',
-                            { self.self_handle : presence },
-                            signature='a{u(uss)}')
+                self.q.dbus_emit(self.object_path,
+                        cs.CONN_IFACE_SIMPLE_PRESENCE, 'PresencesChanged',
+                        { self.self_handle : presence },
+                        signature='a{u(uss)}')
 
             self.q.dbus_return(e.message, signature='')
         else:
@@ -475,6 +562,12 @@ class SimulatedConnection(object):
         except e:
             self.q.dbus_raise(e.message, INVALID_HANDLE, str(e.args[0]))
 
+    def RequestHandles(self, e):
+        htype, idents = e.args
+        self.q.dbus_return(e.message,
+                [self.ensure_handle(htype, i) for i in idents],
+                signature='au')
+
     def GetStatus(self, e):
         self.q.dbus_return(e.message, self.status, signature='u')
 
@@ -489,7 +582,10 @@ class SimulatedConnection(object):
                 status, reason, signature='uu')
         if self.status == cs.CONN_STATUS_CONNECTED and self.has_presence:
             if self.presence[0] == cs.PRESENCE_TYPE_OFFLINE:
-                self.presence = dbus.Struct((cs.PRESENCE_TYPE_AVAILABLE,
+                # "dbus.UInt32" to work around
+                # https://bugs.freedesktop.org/show_bug.cgi?id=69967
+                self.presence = dbus.Struct((
+                    dbus.UInt32(cs.PRESENCE_TYPE_AVAILABLE),
                     'available', ''), signature='uss')
 
             self.q.dbus_emit(self.object_path,
@@ -551,24 +647,76 @@ class SimulatedConnection(object):
                         for channel in channels],
                     signature='a(oa{sv})')
 
+    def get_contact_attributes(self, h, ifaces):
+        id = self.inspect_handles([h])[0]
+        ifaces = set(ifaces).intersection(
+                self.get_contact_attribute_interfaces())
+
+        ret = dbus.Dictionary({}, signature='sv')
+        ret[cs.ATTR_CONTACT_ID] = id
+
+        if cs.CONN_IFACE_ALIASING in ifaces:
+            if h == self.self_handle:
+                ret[cs.ATTR_ALIAS] = self.self_alias
+            else:
+                ret[cs.ATTR_ALIAS] = id
+
+        if cs.CONN_IFACE_AVATARS in ifaces:
+            if h == self.self_handle:
+                if self.avatar is not None and not self.avatar_delayed:
+                    # We just stringify the avatar as the token
+                    # (also, empty avatar => no avatar => empty token).
+                    # This doesn't have the same special case that
+                    # GetKnownAvatarTokens does - if we don't know the
+                    # token yet, we don't wait.
+                    ret[cs.ATTR_AVATAR_TOKEN] = str(self.avatar[0])
+
+        if cs.CONN_IFACE_SIMPLE_PRESENCE in ifaces:
+            if h == self.self_handle:
+                ret[cs.ATTR_PRESENCE] = self.presence
+            else:
+                # stub - MC doesn't care
+                # "dbus.UInt32" to work around
+                # https://bugs.freedesktop.org/show_bug.cgi?id=69967
+                ret[cs.ATTR_PRESENCE] = (dbus.UInt32(cs.PRESENCE_UNKNOWN),
+                        'unknown', '')
+
+        return ret
+
+    def get_contact_attribute_interfaces(self):
+        return set(self.interfaces).intersection(set([
+            cs.CONN_IFACE_ALIASING,
+            cs.CONN_IFACE_AVATARS,
+            cs.CONN_IFACE_SIMPLE_PRESENCE,
+            ]))
+
     def GetContactAttributes(self, e):
-        ret = {}
+        ret = dbus.Dictionary({}, signature='ua{sv}')
 
         try:
             for h in e.args[0]:
-                id = self.inspect_handles(h)[0]
-                ret[dbus.UInt32(h)] = dbus.Dictionary({telepathy.CONN_IFACE + '/contact-id': id},
-                                                      signature='sv')
+                ret[dbus.UInt32(h)] = self.get_contact_attributes(h, e.args[1])
 
-            q.dbus_return(e.message, ret, signature='a{ua{sv}}')
+            self.q.dbus_return(e.message, ret, signature='a{ua{sv}}')
         except e:
             self.q.dbus_raise(e.message, INVALID_HANDLE, str(e.args[0]))
 
+    def GetContactByID(self, e):
+        h = self.ensure_handle(e.args[0])
+        self.q.dbus_return(e.message, h,
+                self.get_contact_attributes(h, e.args[1]), signature='ua{sv}')
+
     def GetAll_Contacts(self, e):
         self.q.dbus_return(e.message, {
-            'ContactAttributeInterfaces': []
+            'ContactAttributeInterfaces':
+                self.get_contact_attribute_interfaces(),
             }, signature='a{sv}')
 
+    def Get_ContactAttributeInterfaces(self, e):
+        self.q.dbus_return(e.message,
+            dbus.Array(self.get_contact_attribute_interfaces(), signature='s'),
+            signature='v')
+
 class SimulatedChannel(object):
     def __init__(self, conn, immutable, mutable={},
             destroyable=False, group=False):
@@ -946,8 +1094,13 @@ def expect_fakecm_connection(q, bus, mc, account, expected_params,
         has_requests=True, has_presence=False, has_aliasing=False,
         has_avatars=False, avatars_persist=True,
         extra_interfaces=[],
-        expect_before_connect=[], expect_after_connect=[],
-        has_hidden=False):
+        expect_before_connect=(), expect_after_connect=(),
+        has_hidden=False,
+        self_ident='myself'):
+    # make (safely) mutable copies
+    expect_before_connect = list(expect_before_connect)
+    expect_after_connect = list(expect_after_connect)
+
     e = q.expect('dbus-method-call', method='RequestConnection',
             args=['fakeprotocol', expected_params],
             destination=cs.tp_name_prefix + '.ConnectionManager.fakecm',
@@ -957,7 +1110,7 @@ def expect_fakecm_connection(q, bus, mc, account, expected_params,
 
     conn = SimulatedConnection(q, bus, 'fakecm', 'fakeprotocol',
                                account.object_path.split('/')[-1],
-            'myself', has_requests=has_requests, has_presence=has_presence,
+            self_ident, has_requests=has_requests, has_presence=has_presence,
             has_aliasing=has_aliasing, has_avatars=has_avatars,
             avatars_persist=avatars_persist, extra_interfaces=extra_interfaces,
             has_hidden=has_hidden)
